Imports System.Drawing.Printing

Public Class Form1
    ' The PrintDocument.
    Private WithEvents m_PrintDocument As New PrintDocument

    ' Information about the paragraphs to print.
    Private Structure ParagraphInfo
        Public FontSize As Integer
        Public Text As String
        Public Sub New(ByVal font_size As Integer, ByVal txt As String)
            FontSize = font_size
            Text = txt
        End Sub
    End Structure

    ' The paragraphs.
    Private m_Paragraphs As Collection
    Private m_ParagraphsToPrint As Collection
    Private m_PagesPrinted As Integer

    ' Load the paragraph info.
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Attach the PrintDocument to the 
        ' PrintDialog and PrintPreviewDialog.
        dlgPrint.Document = m_PrintDocument
        dlgPrintPreview.Document = m_PrintDocument

        ' Make the text to print.
        m_Paragraphs = New Collection
        m_Paragraphs.Add(New ParagraphInfo(45, "23"))
        m_Paragraphs.Add(New ParagraphInfo(27, "Printing"))
        m_Paragraphs.Add(New ParagraphInfo(16, "Visual Basic .NET provides a lot of good tools for printing. String formatting objects let you determine how text is wrapped and truncated if it won't fit in a printing area. Methods provided by Graphics objects let you easily scale, rotate, and translate drawing commands."))
        m_Paragraphs.Add(New ParagraphInfo(16, "The basic process, however, seems somewhat backwards to many programmers. Rather than issuing commands to a printer object, a program responds to requests to draw pages generated by a PrintDocument object."))
        m_Paragraphs.Add(New ParagraphInfo(16, "In some cases, however, generating a printout using only Visual Basic commands can be difficult. The following section explains alternative methods for generating a printout and tells when you might want to use those methods. The rest of the chapter explains basic printing operations in Visual Basic. It shows how to draw graphics and text on the printer, and how to scale and center the results."))
        m_Paragraphs.Add(New ParagraphInfo(27, "How Not To Print"))
        m_Paragraphs.Add(New ParagraphInfo(16, "While Visual Basic provides many tools for arranging graphics on a print out, it does not always provide the best approach to printing. The general approach to printing in Visual Basic requires you to generate each page of output in turn. For simple documents, such as a line drawing containing a few lines and circles, this is not difficult."))
        m_Paragraphs.Add(New ParagraphInfo(16, "Suppose, however, that you want to print several dozen pages of text interspersed with tables and pictures. Figuring out where each page break should end and how the text should flow around the tables and pictures could be a huge undertaking. To really do the job right, you might need to consider orphan lines (when the first line of a paragraph sits at the bottom of a page), widow lines (the last line of a paragraph sits at the top of a page), orphan and widow words (when the first or last word sits on a separate line), inserting extra space between words to make a line look nicer, page numbers, headers and footers, hyphenation, different left and right margins, mirrored margins, the gutter, bulleted lists, indentation and justification, different font sizes and styles, and a host of other issues."))
        m_Paragraphs.Add(New ParagraphInfo(27, "Word Processors"))
        m_Paragraphs.Add(New ParagraphInfo(16, "Word processing and text formatting applications such as Microsoft Word spend a great deal of effort on these issues--effort that you probably don't want to duplicate. In fact, Word is so good at handling these issues that you should consider using it to print your output instead of writing an elaborate Visual Basic program to do it."))
        m_Paragraphs.Add(New ParagraphInfo(16, "If your output is simple text, you can simply place it in a text file and then use Word to open, format, and print it. For printouts that you don't need to generate too frequently and for printouts where the user may want to edit the results before printing anyway, this is a simple flexible solution that doesn't require you to write, debug, and maintain a lot of complicated formatting code."))
        m_Paragraphs.Add(New ParagraphInfo(16, "For more elaborate printouts, programs such as Word may still be useful. Using Visual Studio Tools for Office (VSTO), you can open a Microsoft Word application and control it from your Visual Basic program. Your program can use the Word object model to add text, insert pictures, build tables, set printing page options, and even print the result. You can then save the document for later use or discard it."))
        m_Paragraphs.Add(New ParagraphInfo(16, "Using VSTO, not only can you control Microsoft Word, but you can also use the other Microsoft Office applications. For example, you can load information into Excel so you can use its tools to analyze and graph the data, copy information into Access for analysis by other database applications, or compose email messages in Outlook."))
        m_Paragraphs.Add(New ParagraphInfo(16, "VSTO is relatively complicated and outside of the scope of this book so it isn't described here. For more information, see Microsoft's Web site or a book on Microsoft Office programming."))
    End Sub

    ' Display the preview dialog.
    Private Sub btnPrintPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrintPreview.Click
        dlgPrintPreview.WindowState = FormWindowState.Maximized
        dlgPrintPreview.ShowDialog()
    End Sub

    ' Display the print dialog.
    Private Sub btnPrintDialog_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrintDialog.Click
        If dlgPrint.ShowDialog() = Windows.Forms.DialogResult.OK Then
            dlgPrint.Document.Print()
        End If
    End Sub

    ' Print immediately.
    Private Sub btnPrintNow_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrintNow.Click
        m_PrintDocument.Print()
    End Sub

    ' Get ready to print pages.
    Private Sub m_PrintDocument_BeginPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles m_PrintDocument.BeginPrint
        ' We have not yet printed any pages.
        m_PagesPrinted = 0

        ' Make a copy of the text to print.
        m_ParagraphsToPrint = New Collection
        For Each para_info As ParagraphInfo In m_Paragraphs
            m_ParagraphsToPrint.Add(New ParagraphInfo(para_info.FontSize, para_info.Text))
        Next para_info
    End Sub

    ' Set the margins for the following page.
    Private Sub m_PrintDocument_QueryPageSettings(ByVal sender As Object, ByVal e As System.Drawing.Printing.QueryPageSettingsEventArgs) Handles m_PrintDocument.QueryPageSettings
        ' Use a 1 inch gutter (printer units are 100 per inch).
        Const gutter As Integer = 100

        ' See if the next page will be the first, odd, or even.
        If m_PagesPrinted = 0 Then
            ' The next page is the first.
            ' Increase the left margin.
            e.PageSettings.Margins.Left += gutter
        ElseIf (m_PagesPrinted Mod 2) = 0 Then
            ' The next page will be odd.
            ' Shift the margins right.
            e.PageSettings.Margins.Left += gutter
            e.PageSettings.Margins.Right -= gutter
        Else
            ' The next page will be even.
            ' Shift the margins left.
            e.PageSettings.Margins.Left -= gutter
            e.PageSettings.Margins.Right += gutter
        End If
    End Sub

    ' Print the next page.
    Private Sub m_PrintDocument_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles m_PrintDocument.PrintPage
        ' Increment the page number.
        m_PagesPrinted += 1

        ' Draw the margins (for debugging).
        'e.Graphics.DrawRectangle(Pens.Red, e.MarginBounds)

        ' Print the page number right justified 
        ' in the upper corner opposite the gutter
        ' and outside of the margin.
        Dim the_font As Font
        Dim x As Integer
        Dim string_format As New StringFormat
        the_font = New Font("Times New Roman", _
            20, FontStyle.Regular, GraphicsUnit.Point)

        ' See if this is an odd or even page.
        If (m_PagesPrinted Mod 2) = 0 Then
            ' This is an even page. 
            ' The gutter is on the left.
            x = (e.MarginBounds.Left + e.PageBounds.Left) \ 2
            string_format.Alignment = StringAlignment.Near
        Else
            ' This is an odd page.
            ' The gutter is on the right.
            x = (e.MarginBounds.Right + e.PageBounds.Right) \ 2
            string_format.Alignment = StringAlignment.Far
        End If
        e.Graphics.DrawString(m_PagesPrinted.ToString, _
            the_font, Brushes.Black, x, _
            (e.MarginBounds.Top + e.PageBounds.Top) \ 2, _
            string_format)

        ' Draw the rest of the text left justified,
        ' wrap at words, and don't draw partial lines.
        string_format.Alignment = StringAlignment.Near
        string_format.FormatFlags = StringFormatFlags.LineLimit
        string_format.Trimming = StringTrimming.Word

        ' Draw some text.
        Dim paragraph_info As ParagraphInfo
        Dim ymin As Integer = e.MarginBounds.Top
        Dim layout_rect As RectangleF
        Dim text_size As SizeF
        Dim characters_fitted As Integer
        Dim lines_filled As Integer
        Do While m_ParagraphsToPrint.Count > 0
            ' Print the next paragraph.
            paragraph_info = DirectCast(m_ParagraphsToPrint(1), ParagraphInfo)
            m_ParagraphsToPrint.Remove(1)

            ' Get the font.
            the_font = New Font("Times New Roman", _
                paragraph_info.FontSize, _
                FontStyle.Regular, GraphicsUnit.Point)

            ' Get the area available for this paragraph.
            layout_rect = New RectangleF( _
                e.MarginBounds.Left, ymin, _
                e.MarginBounds.Width, _
                e.MarginBounds.Bottom - ymin)

            ' Make sure the layout rectangle is at least 1 pixel tall
            ' to avoid confusing MeasureString.
            If layout_rect.Height < 1 Then layout_rect.Height = 1

            ' See how big the text will be and 
            ' how many characters will fit.
            text_size = e.Graphics.MeasureString( _
                paragraph_info.Text, the_font, _
                New SizeF(layout_rect.Width, layout_rect.Height), _
                string_format, characters_fitted, lines_filled)

            ' See if any characters will fit.
            If characters_fitted > 0 Then
                ' Draw the text.
                e.Graphics.DrawString(paragraph_info.Text, _
                    the_font, Brushes.Black, _
                    layout_rect, string_format)

                ' Draw a rectangle around the text (for debugging).
                'e.Graphics.DrawRectangle(Pens.Green, _
                '    layout_rect.Left, _
                '    layout_rect.Top, _
                '    text_size.Width, _
                '    text_size.Height)

                ' Increase the location where we can start.
                ' Add a little interparagraph spacing.
                ymin += CInt(text_size.Height + _
                    e.Graphics.MeasureString("M", the_font).Height / 2)
            End If

            ' See if some of the paragraph didn't fit on the page.
            If characters_fitted < Len(paragraph_info.Text) Then
                ' Some of the paragraph didn't fit.
                ' Prepare to print the rest on the next page.
                paragraph_info.Text = paragraph_info.Text. _
                    Substring(characters_fitted)
                m_ParagraphsToPrint.Add(paragraph_info, Before:=1)

                ' That's all that will fit on this page.
                Exit Do
            End If
        Loop

        ' If we have more paragraphs, we have more pages.
        e.HasMorePages = (m_ParagraphsToPrint.Count > 0)
    End Sub

    ' Clean up.
    Private Sub m_PrintDocument_EndPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles m_PrintDocument.EndPrint
        m_ParagraphsToPrint = Nothing
    End Sub
End Class
